import ast

import pandas as pd
import plotly.express as px

import src.MovieTVVis.helper as helper
from src.MovieTVVis.MTVFilter import MTVFilter


class MovieTVVis:

    def __init__(self):
        self.df_original = None
        self.df_processed = None
        self.id_column = None

    def import_csv(self, path: str, sep: str = ",", id_column: str = None):
        """
        Imports a csv file, stores this as a DataFrame, and creates a clean version of the same DataFrame for later use

        :param id_column: Column containing unique values for each observation
        :param path: Path to the csv file
        :param sep: Separator used in csv file
        """
        self.df_original = pd.read_csv(path, sep=sep,quotechar='"')
        if id_column is None:
            self.df_original['ID'] = range(1, len(self.df_original) + 1)
            self.id_column = 'ID'
        else:
            self.id_column = id_column

        list_columns = helper.get_list_column_names(self.df_original.select_dtypes("O"))
        converters = {column: self.safe_literal_eval for column in list_columns}

        self.df_processed = pd.read_csv(path, sep=sep, quotechar='"', converters=converters)
        if id_column is None:
            self.df_processed['ID'] = range(1, len(self.df_processed) + 1)
        else:
            self.id_column = id_column
        self.clean_data(list_columns)

    def safe_literal_eval(self, s):
        try:
            return ast.literal_eval(s)
        except (ValueError, SyntaxError):
            return s

    def get_processed_df(self):
        return self.df_processed

    def get_original_df(self):
        return self.df_original

    def clean_data(self, list_columns):
        """
        Takes the original DataFrame and transforms it into a long format such that it is easier to work with, as well
        as converting "string" columns only containing numeric values into a numeric column.
        """
        self.fix_numeric_columns(inplace=True)
        for col_name in list_columns:
            self.df_processed = self.df_processed.explode(col_name)

    def apply_filter(self, mtv_filter):
        """
        The method applies a MTVFilter to a copy of the processed DataFrame. It modifies a DataFrame based on specified
        criteria. It filters out rows based on an exclusion criteria, and retains rows based on an inclusion criteria.
        No modifications are done to the original DataFrame.

        :param mtv_filter: MTVFilter object with inclusions and/or exclusions
        :return: The filtered DataFrame
        """
        df_working_copy = self.df_processed.copy()

        for feature, values in mtv_filter.get_exclusions().items():
            df_working_copy = helper.filter_out(df_working_copy, feature, values)

        for feature, values in mtv_filter.get_inclusions().items():
            df_working_copy = helper.filter_in(df_working_copy, feature, values)
        return df_working_copy

    def heatmap(self, x, y, mtv_filter=MTVFilter(), relative_values="none", **kwargs):
        """
        Generates a heatmap visualising the relationship between x and y

        :param y: Variable to be visualised along the x-axis
        :param x: Variable to be visualised along the y-axis
        :param mtv_filter: MTVFilter object to be used for filtering values.
        :param relative_values: One of the values "x", "y", "all", or "none".
        This decides which axis is used for calculating percentages
        :param kwargs: Optional arguments to be passed to Plotly
        :return: A Plotly figure object.
        """
        df_filtered = self.apply_filter(mtv_filter=mtv_filter)
        df_filtered.drop_duplicates(
            subset=[self.id_column, y, x],
            keep="last",
            inplace=True
        )

        df_grouped = df_filtered.groupby([y, x]).count().reset_index()
        heatmap_data = df_grouped.pivot_table(index=y, columns=x, values=self.id_column)

        if relative_values == "x":
            heatmap_data = heatmap_data.apply(lambda z: z / z.sum(), axis=0)
        elif relative_values == "y":
            heatmap_data = heatmap_data.apply(lambda z: z / z.sum(), axis=1)
        elif relative_values == "all":
            heatmap_data = heatmap_data / heatmap_data.sum().sum()

        text_auto = {
            "x": ".0%",
            "y": ".0%",
            "all": ".0%",
            "none": True,

        }

        title = kwargs.pop("title", f"Number of screen content per {y} / {x}")
        template = kwargs.pop("template", "none")
        text_auto = kwargs.pop("text_auto", text_auto[relative_values])
        color_continuous_scale = kwargs.pop("color_continuous_scale", "Blues")

        fig = px.imshow(
            heatmap_data,
            template=template,
            text_auto=text_auto,
            color_continuous_scale=color_continuous_scale,
            title=title,
            **kwargs
        )

        fig.update_layout(
            coloraxis_showscale=False
        )

        return fig

    def pie(self, target, mtv_filter=MTVFilter(), relative_values=True, **kwargs):
        """
        Generates a pie plot visualising the distribution of the target variable

        :param target: The variable to be visualised
        :param mtv_filter: MTVFilter object to be used for filtering values.
        :param relative_values: True or False depending on whether values should be displayed as frequencies or relative
        frequencies
        :param kwargs: Optional arguments to be passed to Plotly
        :return: A Plotly figure object.
        """
        df = self.apply_filter(mtv_filter)
        df.drop_duplicates(
            subset=[self.id_column, target],
            keep="last",
            inplace=True
        )

        df_grouped = df.groupby([target])[target].count().reset_index(name="Count")

        template = kwargs.pop("template", "none")
        title = kwargs.pop("title", f"Distribution of screen content per {target}")

        fig = px.pie(
            data_frame=df_grouped,
            names=df_grouped[target],
            values=df_grouped["Count"],
            template=template,
            title=title,
            **kwargs
        )

        textinfo = [" + value", " + percent"]
        texttemplate = ["%{value}", "%{percent:.0%}"]

        fig.update_traces(
            textinfo="label" + textinfo[relative_values],
            textposition="outside",
            texttemplate="%{label} " + texttemplate[relative_values]
        )
        return fig

    def bar(self, target, mtv_filter=MTVFilter(), relative_values=True, **kwargs):
        """
        Generates a bar plot visualising the distribution of the target variable

        :param target: The variable to be visualised
        :param mtv_filter: MTVFilter object to be used for filtering values.
        :param relative_values: True or False depending on whether values should be displayed as frequencies or relative
        frequencies
        :param kwargs: Optional arguments to be passed to Plotly
        :return: A Plotly figure object.
        """
        df = self.apply_filter(mtv_filter)
        df.drop_duplicates(
            subset=[self.id_column, target],
            keep="last",
            inplace=True
        )

        df_grouped = df.groupby([target])[target].count().reset_index(name="Frequencies")
        df_grouped.sort_values(inplace=True, ascending=False, by="Frequencies")
        df_grouped["Relative frequencies"] = df_grouped["Frequencies"] / df_grouped["Frequencies"].sum()

        text_auto = kwargs.pop("text_auto", [True, ".0%"][relative_values])
        template = kwargs.pop("template", "none")
        title = kwargs.pop("title", f"Distribution of screen content per {target}")

        fig = px.bar(
            data_frame=df_grouped,
            x=target,
            y="Relative frequencies" if relative_values else "Frequencies",
            text_auto=text_auto,
            template=template,
            title=title,
            **kwargs
        )

        fig.update_traces(
            textposition="outside",
            cliponaxis=False
        )

        return fig

    def line(self, x, target, mtv_filter=MTVFilter(), relative_values=False, **kwargs):
        """
        Generates a line plot visualisation where x represents the x-axis, and target represents the different lines

        :param x: Variable to be used as the x-axis
        :param target: Variable to be used for individual lines
        :param mtv_filter: MTVFilter object to be used for filtering values.
        :param relative_values: True or False depending on whether values should be displayed as frequencies or relative
        frequencies
        :param kwargs: Optional arguments to be passed to Plotly
        :return: A Plotly figure object.
        """
        df = self.apply_filter(mtv_filter)
        df.drop_duplicates(
            subset=[self.id_column, x, target],
            keep="last",
            inplace=True
        )

        df_grouped = df.groupby([target, x])[target].count().reset_index(name="Frequencies")
        df_grouped.sort_values(inplace=True, ascending=False, by=x)
        df_grouped["Relative frequencies"] = df_grouped.groupby("Year")["Frequencies"].transform(lambda z: z / z.sum())

        template = kwargs.pop("template", "none")
        markers = kwargs.pop("markers", True)
        title = kwargs.pop("title", f"screen content per {x} for {target}")

        fig = px.line(
            df_grouped,
            x=x,
            y="Relative frequencies" if relative_values else "Frequencies",
            color=target,
            template=template,
            markers=markers,
            title=title,
            **kwargs
        )

        fig.update_yaxes(
            tickformat=".0%" if relative_values else None
        )
        return fig

    def scatter(self, x, y, mtv_filter=MTVFilter(), trendline=False, **kwargs):
        """
        Generates a scatter plot visualising the numerical variables x against y

        :param x: Name of the variable to be used as x-axis
        :param y: Name of the variable to be used as y-axis
        :param mtv_filter: MTVFilter obliject to be used for filtering values.
        :param trendline: True if the scatterplot should include an ordinary least squares trendline. False otherwise
        :param kwargs: Optional arguments to be passed to Plotly
        :return: A Plotly figure object
        """
        df = self.apply_filter(mtv_filter)
        df.drop_duplicates(
            subset=[self.id_column],
            keep="last",
            inplace=True
        )

        title = kwargs.pop("title", f"Relationship between {x} and {y}")
        trendline_variable = kwargs.pop("trendline", [None, "ols"][trendline])

        fig = px.scatter(
            df,
            x=x,
            y=y,
            trendline=trendline_variable,
            title=title,
            template="none",
            **kwargs
        )
        return fig

    def combine_features(self, feature_a, feature_b, new_feature_name):
        """
        Combine two feature columns into a new column.
        :param feature_a: Name of the first column to be combined
        :param feature_b: Name of the second column to be combined
        :param new_feature_name: Name of the new column name

        :return: A new DataFrame with the combined features if inplace = False, otherwise it changes the internal
        DataFrame and returns this.
        """
        df = self.df_processed.copy()
        df[new_feature_name] = df[feature_a]

        df_copied_rows = df[df[feature_b].notna()].copy()
        df_copied_rows[new_feature_name] = df_copied_rows[feature_b]
        df_copied_rows = df_copied_rows.drop(columns=[feature_b])

        df_combined = pd.concat([df, df_copied_rows], ignore_index=True)
        df_combined.drop_duplicates(inplace=True)
        df_combined.sort_values(by=self.id_column, inplace=True)
        self.df_processed = df_combined

        return df_combined

    def fill_nas(self, fill_with="", inplace=False):
        """
        Fills all NA values in all columns which are of type "object"
        :param fill_with: value to replace NA's
        :param inplace: True if the operation is to be executed on the internal DataFrame. False if a copy is created
        and returned
        :return: A DataFrame with NA's replaced. The DataFrame is the internal DataFrame if inplace = True, otherwise it
        returns a copy of the internal DataFrame
        """
        """

        :param fill_with: The value to fill NA's with
        :param inplace: True if the operation should be done on the internal object. 
        If False, a new DataFrame is returned.
        :return:
        """
        df = self.df_processed.copy()
        for feature in df.select_dtypes(include="O").columns:
            df[feature].fillna(fill_with, inplace=True)
        if inplace:
            self.df_processed = df
            return self.df_processed
        return df

    def fix_numeric_columns(self, inplace=False):
        """
        Finds the columns which should be numeric according to get_numeric_object_columns and transforms them to numeric.
        :param inplace: True if action should be done on the internal dataframe
        :return: None if inplace = True, converted dataframe if inplace= False
        """
        numeric_column_names = self.get_numeric_object_columns()
        return self.transform_columns_to_numeric(numeric_column_names, inplace=inplace)

    def get_numeric_object_columns(self):
        """
        Searches through the dataframe for columns which can be converted to numeric
        :return: list of column names which can be converted to numeric.
        """
        columns = self.df_processed.select_dtypes(include="object").columns
        return [column for column in columns if helper.column_is_numeric(self.convert_to_numeric(column))]

    def transform_columns_to_numeric(self, col_names, inplace):
        """
        Converts columns which can be converted to numeric, into numeric.
        :param col_names: Names of columns which should be converted
        :param inplace: True if action is taken on objects dataframe, False if a new dataframe should be returned
        :return: None if inplace = True, and a new dataframe if inplace = False
        """
        if inplace:
            self.df_processed[col_names] = self.df_processed[col_names].apply(lambda col: helper.to_numeric(col))
        else:
            df_copy = self.df_processed.copy()
            df_copy[col_names] = df_copy[col_names].apply(lambda col: helper.to_numeric(col))
            return df_copy

    def convert_to_numeric(self, column_name):
        """
        Replaces commas with periods and returns a column where the values have been converted to numeric.

        :param column_name: name of column to be converted
        :return: converted column
        """
        return pd.to_numeric(helper.to_numeric(self.df_processed[column_name]))

    def describe(self):
        """
        Prints a description of what columns and unique values are present in the DataFrame
        :return: None
        """
        columns = self.df_processed.columns.tolist()
        unique_values = [pd.unique(self.df_processed[column]).tolist() for column in columns]
        print("COLUMN NAMES AND UNIQUE VALUES\n")
        for column, values in zip(columns, unique_values):
            print(column)
            print(values)
            print()

    def print_unique_columns(self):
        """
        Prints a list of what columns are present in the DataFrame
        :return: None
        """
        print("COLUMNS PRESENT\n")
        for column in self.df_processed.columns:
            print(column)

    def print_unique_values(self, column_name):
        for value in pd.unique(self.df_processed[column_name]):
            print(value)



class NorwayTop10:
    def __init__(self):
        self.df = None

    def findTop10(self, dataframe, country, year, film=True, tv=True):
        """
        The function should find the most popular tv shows and films by giving 10 points to a film or TV if they are at the top of the top 10 list, and
        one point if they are tenth. Then we sum this together for the whole year for each movie and tv show. 
        This function takes in a dataframe from the netflix data, a country and year to sort the data on and a boolean value for film and tv.
        The boolean value for both film and tv is as standard set to true, meaning the top 10 list we create contains both films and tv shows.
        If we want to look at only films we set tv to False and if we only want tv we set film to false. If both are false we return an empty dictionary.
        The first part of the function checks given statements and make sures that the given input to the function is correct, and if the input in year or country
        is not correct it tries to correct it if possible. If the given country or year is not in the dataframe it raises a valueError.
        The next part of the function sorts on the given country and year and creates a copy of the dataframe to work on, to not change the given one.
        Then based on the boolean values for film and tv we sort the dataframe on movies or tv shows. 
        Next we apply a lambda function where we give 10 points to first place on the weekkly_rank and one point to last place.
        Then we groupby the show title and aggregate the points column we created in the last part by sum.
        Next we sort the values by points and get out the top 10 movies and tv shows for the given year and country.
        Lastly we convert the show_title and the points score to a dictionary displaying the shows in the top 10 and the points they got. 
        """
        if not film and not tv:
            return {}
        #Capitalize the first letter of country to extract country data from df
        country = country.capitalize()

        if not isinstance(year, str):
            year = str(year)

        if (country not in dataframe['country_name'].unique()):
            raise ValueError('Country not in the given dataset, check spelling.')

        if str(year) not in dataframe['week'].str.slice(0, 4).unique():
            raise ValueError('Year not in the dataset, check years included in the given dataset.')

        # Filter the dataframe based on the given country and year
        df_filtered = dataframe[
            (dataframe['country_name'] == country) & (dataframe['week'].str.startswith(str(year)))].copy()

        # Filter based on the category (Film or TV Shows)
        if film and not tv:
            df_filtered = df_filtered[df_filtered['category'] == 'Films']
        elif tv and not film:
            df_filtered = df_filtered[df_filtered['category'] != 'Films'] 

        # Assign points based on weekly rank
        df_filtered['points'] = df_filtered['weekly_rank'].apply(lambda x: 11 - x)

        # Group by show title and sum the points
        df_grouped = df_filtered.groupby('show_title').agg({'points': 'sum'}).reset_index()

        # Sort by points and get top 10
        top10_shows = df_grouped.sort_values(by='points', ascending=False).head(10)

        # Convert dataframe to dictionary
        result = top10_shows.set_index('show_title')['points'].to_dict()

        return result

    def cumulativeWeeksTop10(self, dataframe, country, year, film=True, tv=True):
        """
        The function should return the top 10 movies or tv shows that has been the most cumulative weeks in the top 10 on netflix in a given year.
        This function takes in a dataframe, a country and year to sort the data on and a boolean value for tv and film.
        The boolean value for both film and tv is as standard set to true, meaning the top 10 list we create contains both films and tv shows.
        If we want to look at only films we set tv to False and if we only want tv we set film to false. If both are false we return an empty dictionary.
        The first part of the function checks given statements and make sures that the given input to the function is correct, and if the input in year or country
        is not correct it tries to correct it if possible. If the given country or year is not in the dataframe it raises a valueError.
        The next part of the function sorts on the given country and year and creates a copy of the dataframe to work on, to not change the given one.
        Then based on the boolean values for film and tv we sort the dataframe on movies or tv shows. 
        Next we keep only the rows with the maximum value for cumulative weeks in top 10 and then filter the dataframe on these indexes.
        Then we sort the given dataframe based on weeks in top 10 and get the top 10 movies or tv shows and creates a dictionary with this information.
        """
        if not film and not tv:
            return {}
        # Capitalizes the first letter of country to extract country data from df
        country = country.capitalize()

        if not isinstance(year, str):
            year = str(year)

        if (country not in dataframe['country_name'].unique()):
            raise ValueError('Country not in the given dataset, check spelling.')
        if year not in dataframe['week'].str.slice(0, 4).unique():
            raise ValueError('Year not in the dataset, check years included in the given dataset.')

        # Filter the dataframe based on the given country and year
        df_filtered = dataframe[
            (dataframe['country_name'] == country) & (dataframe['week'].str.startswith(str(year)))].copy()

        # Filter based on the category (Film or TV Shows)
        if film and not tv:
            df_filtered = df_filtered[df_filtered['category'] == 'Films']
        elif tv and not film:
            df_filtered = df_filtered[df_filtered['category'] != 'Films'] 
        indexes = df_filtered.groupby('show_title')['cumulative_weeks_in_top_10'].idxmax()
        # Filter dataframe using the indices of max cumulative weeks in top 10
        df_max_cumulative = df_filtered.loc[indexes]
        top10shows = df_max_cumulative.sort_values(by='cumulative_weeks_in_top_10', ascending=False).head(10)
        result = top10shows.set_index('show_title')['cumulative_weeks_in_top_10'].to_dict()

        return result



import requests
from bs4 import BeautifulSoup


def get_most_popular_title_urls(url: str) -> list:
    """
    Takes in a JustWatch search page and returns a list of urls to the titles on the page

    :param url: URL to a JustWatch search page
    :return: A list of urls to the titles on the page
    """
    response = requests.get(url)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        a_tags = soup.find_all("a", class_="title-list-grid__item--link")
        hrefs = ["https://www.justwatch.com" + a_tag.get("href") for a_tag in a_tags]
        return hrefs


def get_popularity_list_urls(providers: list, years: list) -> dict:
    """
    Creates a list of urls using the provided parameters for the JustWatch popularity website

    :param providers: List of providers to include in the url
    :param years: List of years to include in the url
    :return: List of urls
    """
    popularity_list_urls = {}
    for provider in providers:
        for year in years:
            url = F"https://www.justwatch.com/no/provider/{provider}/tv-series?exclude_genres=doc&exclude_genres=fml&exclude_genres=rly&release_year_from={year}&release_year_until={year}"
            popularity_list_urls[(provider, year)] = url
    return popularity_list_urls


def get_title(object_url: str) -> str | None:
    """
    Retrieves the title of a title on JustWatch

    :param object_url: url of a title on JustWatch
    :return: The title for the given url
    """
    response = requests.get(object_url)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        tags = soup.find_all("h1")
        name = tags[0].text.strip()
        return name
    else:
        return None


def get_detail_infos(object_url: str, heading_of_interest: str) -> list:
    response = requests.get(object_url)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        names = soup.find_all("h3", class_="detail-infos__subheading")
        tags = soup.find_all("div", class_="detail-infos__value")
        headings = [x.text.strip() for x in names]

        if heading_of_interest in headings:
            index = headings.index(heading_of_interest)
            values = tags[index].text.strip().split(", ")
            return [value.strip() for value in values]

    return []


def get_prod_country(object_url: str) -> list:
    """
    Retrieves the production country / countries of a title on JustWatch
    :param object_url: url of a title on JustWatch
    :return: a list of production countries for the given url
    """
    return get_detail_infos(object_url, "Production country")


def get_genre(object_url: str) -> list:
    """
    Retrieves the genre / genres of a title on JustWatch
    :param object_url: url of a title on JustWatch
    :return: a list of genres for the given url
    """
    return get_detail_infos(object_url, "Genres")


def get_director(object_url: str) -> list:
    return get_detail_infos(object_url, "Director")


def get_synopsis(object_url: str) -> str | None:
    """
    Retrieves the synopsis of a title on JustWatch

    :param object_url: url of a title on JustWatch
    :return: The synopsis for the given url
    """
    response = requests.get(object_url)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        synopsis = soup.find_all("p", class_="text-wrap-pre-line mt-0")
        if len(synopsis) > 0:
            return synopsis[0].text.strip()
    else:
        return None


def get_cast(object_url):
    """
    Retrieves the cast of a title on JustWatch

    :param object_url: url of a title on JustWatch
    :return: The cast for the given url
    """
    response = requests.get(object_url)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        cast = soup.find_all("span", class_="title-credit-name")
        cast = [c.text.strip() for c in cast]
        return cast
    else:
        return None


def get_languages(tmdb_details):
    """
    Finds the different languages of a movie or tv show, using the details from a search to TMDB's api.
    """
    languages = tmdb_details.get("languages", [])
    return languages


def get_original_language(tmdb_details):
    """
    Finds the original language of a movie or tv show, using the details from a search to TMDB's api.
    """
    original_language = tmdb_details.get("original_language", "")
    return original_language


def get_production_companies(tmdb_details):
    """
    Finds the production companies involved in creating the tv show or movie, using the details from a search to TMDB's api.
    """
    production_companies = tmdb_details.get("production_companies", [])
    return [company["name"] for company in production_companies]


def get_tmdb_details(title, api_key, is_movie=False):
    """
    Searches TMDB's api for the given movie title and gets important information about the movie or tv show.
    To get this function to work you need to request TMDB for your own api key, and then input this key as a string to this function.
    """
    base_url = "https://api.themoviedb.org/3"
    search_url = f"{base_url}/search/movie" if is_movie else f"{base_url}/search/tv"
    params = {
        "api_key": api_key,
        "query": title
    }
    response = requests.get(search_url, params=params)
    if response.status_code == 200:
        results = response.json().get("results")
        if results:
            # Assuming the first result is the most relevant
            id = results[0]["id"]
            details_url = f"{base_url}/movie/{id}" if is_movie else f"{base_url}/tv/{id}"
            details_response = requests.get(details_url, params={"api_key": api_key})
            if details_response.status_code == 200:
                return details_response.json()
    return {"original_language": [],
            "languages": [],
            "production_companies": []}


class Scraper:
    def __init__(self):
        self.metadata = {
            "Rank": [],
            "Title": [],
            "ProductionCountry": [],
            "Genre": [],
            "Cast": [],
            "Synopsis": [],
            "Original language": [],
            "Languages": [],
            "Production Companies": [],
            "Director": []
        }

    def get_jw_metadata(self, object_url, api_key):
        """
        Retrieves the metadata for a title on JustWatch and adds it to the Scraper object
        :param api_key:
        :param object_url:
        :return:
        """
        self.metadata["Rank"].append(len(self.metadata["Rank"]) + 1)
        self.metadata["Title"].append(get_title(object_url))
        self.metadata["ProductionCountry"].append(get_prod_country(object_url))
        self.metadata["Genre"].append(get_genre(object_url))
        self.metadata["Cast"].append(get_cast(object_url))
        self.metadata["Synopsis"].append(get_synopsis(object_url))
        self.metadata["Director"].append(get_director(object_url))

        tmdb_details = get_tmdb_details(get_title(object_url), api_key)
        self.metadata["Original language"].append(get_original_language(tmdb_details))
        self.metadata["Languages"].append(get_languages(tmdb_details))
        self.metadata["Production Companies"].append(get_production_companies(tmdb_details))

    def get_metadata(self):
        return self.metadata



import pandas as pd


class MTVFilter:

    def __init__(self) -> None:
        self.exclusions = {}
        self.inclusions = {}

    def __add_to_filter(self, exclusion: bool, column_name: str, values: list) -> None:
        if exclusion:
            self.exclusions[column_name] = values
        else:
            self.inclusions[column_name] = values

    def add_exclusion(self, column_name: str, values: list) -> None:
        """
        Adds an exclusions to the filter

        :param column_name: Name of the column with values to be excluded
        :param values: to be excluded
        :return: None
        """
        self.__add_to_filter(exclusion=True, column_name=column_name, values=values)

    def add_inclusion(self, column_name: str, values: list) -> None:
        """
        Adds an inclusion to the filter

        :param column_name: Name of the column with values to be included
        :param values: to be included
        :return: None
        """
        self.__add_to_filter(exclusion=False, column_name=column_name, values=values)

    def add_exclusions(self, filter_dict: dict) -> None:
        """
        Adds multiple exclusions to the filter in a single operation
        :param filter_dict: Dictionary containing column names as keys and list of values as values
        :return: None
        """
        for column, values in filter_dict.items():
            self.add_exclusion(column, values)

    def add_inclusions(self, filter_dict: dict) -> None:
        """
        Adds multiple inclusions to the filter in a single operation
        :param filter_dict: Dictionary containing column names as keys and list of values as values
        :return: None
        """
        for column, values in filter_dict.items():
            self.add_inclusion(column, values)

    def remove_exclusion(self, column_name: str) -> None:
        """
        Removes the exclusion of a single column from the filter. All values from the column are removed from the filter.
        :param column_name: to be removed from the filter
        :return: None
        """
        del self.exclusions[column_name]

    def remove_inclusion(self, column_name: str) -> None:
        """
        Removes the inclusion of a single column from the filter. All values from the column are removed from the filter.
        :param column_name: to be removed from the filter
        :return: None
        """
        del self.inclusions[column_name]

    def remove_exclusions(self, column_names: list) -> None:
        """
        Removes the exclusion of multiple columns from the filter. All values from the columns are removed from the filter
        :param column_names: to be removed from the filter
        :return: None
        """
        for column_name in column_names:
            self.remove_exclusion(column_name)

    def remove_inclusions(self, column_names: list) -> None:
        """
        Removes the inclusions of multiple columns from the filter. All values from the columns are removed from the filter
        :param column_names: to be removed from the filter
        :return: None
        """
        for column_name in column_names:
            self.remove_inclusion(column_name)

    def get_exclusions(self) -> dict:
        return self.exclusions

    def get_inclusions(self) -> dict:
        return self.inclusions

    def get_filter(self) -> (dict, dict):
        return self.get_exclusions(), self.get_inclusions()

    def reset(self) -> None:
        """
        Resets the entire filter to its default state
        :return: None
        """
        self.reset_exclusions()
        self.reset_inclusions()

    def reset_exclusions(self) -> None:
        """
        Resets the exclusion filter to its default state
        :return: None
        """
        self.exclusions.clear()

    def reset_inclusions(self) -> None:
        """
        Resets the inclusion filter to its default state
        :return: None
        """
        self.inclusions.clear()

    def describe(self) -> None:
        """
        Prints out a description of which columns and belonging values are to be excluded, and likewise for inclusions
        :return: None
        """
        df_exclusions = pd.DataFrame(data={
            "Columns": list(self.exclusions.keys()),
            "Values": list(self.exclusions.values())
        })

        df_inclusions = pd.DataFrame(data={
            "Columns": list(self.inclusions.keys()),
            "Values": list(self.inclusions.values())
        })

        print("EXCLUSIONS")
        if len(self.exclusions) == 0:
            print("None set")
        else:
            print(df_exclusions.set_index("Columns"))
        print("\nINCLUSIONS")
        if len(self.inclusions) == 0:
            print("None set")
        else:
            print(df_inclusions.set_index("Columns"))



import dash
from dash import html, dcc
from dash.dependencies import Input, Output
import plotly.graph_objs as go
from src.MovieTVVis import MovieTVVis
import plotly.express as px

app = dash.Dash(__name__, suppress_callback_exceptions=True)
mtv = MovieTVVis.MovieTVVis()
mtv.import_csv("data/movies_dummy_data_lists.csv", sep=';')
available_variables = ["ID", "MovieName", "Year", "Genre", "ProductionCountry", "ProductionCompany", "IMDBScore", "Runtime", "StreamingService", "RentBuyService"]

app.layout = html.Div([
    dcc.Tabs([
        dcc.Tab(label='Heatmap', children=[
            dcc.Dropdown(
                id='heatmap-x-variable',
                options=[{'label': var, 'value': var} for var in available_variables],
                placeholder='x'
            ),
            dcc.Dropdown(
                id='heatmap-y-variable',
                options=[{'label': var, 'value': var} for var in available_variables],
                placeholder='y'
            ),
            dcc.RadioItems(
                id='relative-values',
                options=[
                    {'label': 'None', 'value': 'none'},
                    {'label': 'X-axis', 'value': 'x'},
                    {'label': 'Y-axis', 'value': 'y'},
                    {'label': 'All', 'value': 'all'}
                ],
            ),
            dcc.Graph(id='heatmap-output')
            
        ]),
        dcc.Tab(label='Pie Chart', children=[
            dcc.Dropdown(
                id='pie-target-variable',
                options=[{'label': var, 'value': var} for var in available_variables],
                placeholder='Target variable'
            ),
            dcc.Dropdown(
                id='pie-relative-values',
                options=[{'label': 'True', 'value': True}, {'label': 'False', 'value': False}],
                placeholder='Relative values'
            ),
            dcc.Graph(id='pie-chart-output')

        ]),
        dcc.Tab(label='Bar Chart', children=[
            dcc.Dropdown(
                id='bar-target-variable',
                options=[{'label': var, 'value': var} for var in available_variables],
                placeholder='Target variable'
            ),
            dcc.Dropdown(
                id='bar-relative-values',
                options=[{'label': 'True', 'value': True}, {'label': 'False', 'value': False}],
                placeholder='Relative values'
            ),
            dcc.Graph(id='bar-chart-output')

        ]),
        dcc.Tab(label='Line Chart', children=[
            dcc.Dropdown(
                id='line-x-variable',
                options=[{'label': var, 'value': var} for var in available_variables],
                placeholder='x'
            ),
            dcc.Dropdown(
                id='line-target-variable',
                options=[{'label': var, 'value': var} for var in available_variables],
                placeholder='Target variable'
            ),
            dcc.Dropdown(
                id='line-relative-values',
                options=[{'label': 'True', 'value': True}, {'label': 'False', 'value': False}],
                placeholder='Relative values'
            ),
            dcc.Graph(id='line-chart-output')

        ]),
        dcc.Tab(label='Scatter Plot', children=[
            dcc.Dropdown(
                id='scatter-x-variable',
                options=[{'label': var, 'value': var} for var in available_variables],
            ),
            dcc.Dropdown(
                id='scatter-y-variable',
                options=[{'label': var, 'value': var} for var in available_variables],
            ),
            dcc.Dropdown(
                id='scatter-trendline',
                options=[{'label': 'True', 'value': True}, {'label': 'False', 'value': False}],
            ),
            dcc.Graph(id='scatter-plot-output')

        ])
    ]),
])



@app.callback(
    Output('heatmap-output', 'figure'),
    [
        Input('heatmap-x-variable', 'value'),
        Input('heatmap-y-variable', 'value'),
        Input('relative-values', 'value')
    ]
)
def update_heatmap(x_var, y_var, relative_value):
    fig = mtv.heatmap(x=x_var, y=y_var, relative_values=relative_value)
    return fig


@app.callback(
    Output('pie-chart-output', 'figure'),
    Input('pie-target-variable', 'value'),
    Input('pie-relative-values', 'value')   
)
def update_pie_chart(target_var, relative_value):
    fig = mtv.pie(target=target_var, relative_values=relative_value)
    return fig


@app.callback(
    Output('bar-chart-output', 'figure'),
    [
        Input('bar-target-variable', 'value'),
        Input('bar-relative-values', 'value')
    ]
)
def update_bar_chart(target_var, relative_value):
    fig = mtv.bar(target=target_var, relative_values=relative_value)
    return fig


@app.callback(
    Output('line-chart-output', 'figure'),
    [
        Input('line-x-variable', 'value'),
        Input('line-target-variable', 'value'),
        Input('line-relative-values', 'value')
    ]
)
def update_line_chart(x_var, target_var, relative_value):
    fig = mtv.line(x=x_var, target=target_var, relative_values=relative_value)
    return fig


@app.callback(
    Output('scatter-plot-output', 'figure'),
    [
        Input('scatter-x-variable', 'value'),
        Input('scatter-y-variable', 'value'),
        Input('scatter-trendline', 'value'),
    ]
)
def update_scatter_plot(x_var, y_var, trendline):
    fig = mtv.scatter(x=x_var, y=y_var, trendline=trendline)
    return fig


if __name__ == '__main__':
    app.run_server(debug=True)


import pandas as pd


def column_is_numeric(column: pd.DataFrame):
    """
    Calculates the percentage of non-NA values in a column which are numeric, and returns True if this value is 100%

    :param column: Pandas column
    :return: True if all values are numeric, not counting nan values. False otherwise.
    """
    converted_proportion = column.notna().mean()
    return converted_proportion == 1


def to_numeric(column: pd.DataFrame):
    """
    Replaces commas with periods, and converts the given column to numeric

    :param column: To be converted to numeric values
    :return: converted column with numeric values
    """
    return pd.to_numeric(column.str.replace(",", "."), errors="coerce")


def filter_in(df: pd.DataFrame, variable: str, values: list):
    """
    Filter a DataFrame to include only specified values in a particular column.

    :param df: The input DataFrame to be filtered
    :param variable: Name of column in df which is to be filtered
    :param values: List of values which should be retained in the DataFrame.
    :return: The filtered DataFrame containing only rows with values in filter_values for the column filter_variable
    """
    if values is not None:
        return df[df[variable].isin(values)]
    return df


def filter_out(df, variable, values):
    """
    Filter a DataFrame to exclude all values specified in a particular column.

    :param df: The input DataFrame to be filtered
    :param variable: Name of column in df which is to be filtered
    :param values: List of values which should be excluded in the DataFrame.
    :return: The filtered DataFrame containing only rows with values not in filter_values for the column filter_variable
    """
    if values is not None:
        return df[~df[variable].isin(values)]
    return df


def column_values_are_lists(df, column_name):
    """
    Checks if all non-Na values in a DataFrame column are enclosed with square brackets and returns True if this is
    the case

    :param df: Dataframe with column to be checked
    :param column_name: Name of column in Dataframe
    :return: True if all values in the specified column are "list-like"
    """
    df_column = df[column_name]
    return df_column.apply(lambda x: x.startswith("[") and x.endswith("]") if pd.notna(x) else True).all()


def get_list_column_names(df):
    """
    Returns a list of the names of columns in a DataFrame which are "list-like"

    :param df: DataFrame to be checked
    :return: list of column names which are "list-like"
    """
    column_names = []
    for col_name in df.columns:
        if column_values_are_lists(df, col_name):
            column_names.append(col_name)

    return column_names



from setuptools import setup, find_packages

setup(
    name='digiscreens_uib',
    version='1.2',
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    package_data={'data': ['data/*.csv', 'data/*.xlsx', 'data/scraping/*.csv']},
    install_requires=[
        'numpy',
        'pandas',
        'plotly'
    ],
    include_package_data=True,
    author='Digiscreens_team_uib',
    description='Digiscreens package',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    license='MIT',
    url='https://git.app.uib.no/ii/inf219/23h/digiscreens'
)